package be.iminds.ilabt.jfed.highlevel.util;

import be.iminds.ilabt.jfed.util.common.IOUtils;
import net.schmizz.sshj.SSHClient;
import net.schmizz.sshj.connection.channel.direct.Session;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

/**
 * User: twalcari
 * Date: 6/3/2016
 * Time: 15:41
 */
public class ExternalFileUtil {
    private static final Logger LOG = LoggerFactory.getLogger(ExternalFileUtil.class);

    @Nonnull private final SSHClient ssh;
    @Nullable private String homedir;

    public ExternalFileUtil(@Nonnull SSHClient ssh) {
        this.ssh = ssh;
    }

    @Nonnull
    public String getFolder(@Nonnull String fullPath) {
        int separatorLocation = fullPath.lastIndexOf('/');

        String folder;
        if (separatorLocation != -1) {
            folder = fullPath.substring(0, separatorLocation);
        } else {
            LOG.warn("Path {} is an invalid file path", fullPath);
            throw new IllegalArgumentException("Path '" + fullPath + "' is not a valid file path");
        }

        if (folder.contains("~")) {
            if (homedir == null)
                retrieveHomedir();

            assert homedir != null;
            folder = folder.replace("~", homedir);
        }

        return folder;
    }

    @Nonnull
    public String getHomeDir() {
        if (homedir == null)
            retrieveHomedir();
        return homedir;
    }

    @Nonnull
    public String retrieveHomedir() {
        try (Session session = ssh.startSession()) {
            //pwd returns current dir, but after SSH login, that is almost always the homedir of the user
            //Alternatives:
            //    echo ~
            // or
            //    echo $HOME
            Session.Command command = session.exec("pwd");
            command.join();
            if (command.getExitStatus() == 0) {
                try (BufferedReader buffer = new BufferedReader(new InputStreamReader(command.getInputStream()))) {
                    homedir = buffer.readLine();
                }
                LOG.info("Retrieved  homedir '{}' from {}", homedir, ssh.getRemoteHostname());

                if (homedir == null)
                    throw new RuntimeException("Could not retrieve the homefolder on the external system (got null)");
            } else {
                LOG.error("Error while executing 'pwd' on {}", ssh.getRemoteHostname());
                throw new RuntimeException("Error while executing 'pwd' on "+ ssh.getRemoteHostname());
            }
        } catch (IOException ex) {
            throw new RuntimeException("Could not retrieve the homefolder on the external system", ex);
        }

        return homedir;
    }

    /**
     * @param fullPath
     * @return the filename of the file specified in fullPath (removes leading directories, and keeps any extension)
     */
    @Nonnull
    public static String getBasename(@Nonnull String fullPath) {
        return IOUtils.getUnixBasename(fullPath);
    }

    /**
     * @param fullPath
     * @return the path of the file specified in fullPath (removes filename part, leaving all directoriess)
     */
    @Nonnull
    public static String getPathOnly(@Nonnull String fullPath) {
        return IOUtils.getUnixPathOnly(fullPath);
    }
}
